#!/usr/bin/perl

use strict;
use warnings;

BEGIN {
    if ($0 =~ m|^(.*)/[^/]+$|) {
        push(@INC, $1);
    }
}

my $usage = "Usage: $0 outputdir\n";
my $dir = shift(@ARGV) or die $usage;
die $usage unless ($#ARGV < 0);
die $usage unless (-d $dir);

use HPSMSOAP;
use HTML::Template;

my $site = 'http://bp4smdemo.hpsweducation.com:13080/SM/7';
my $user = 'falcon';
my $password = 'orange perfection';

my %expected = ();

foreach my $i (qw(answer author
		  categories cause creationdate
		  doctype
		  error
		  fix
		  id
		  locale
		  question
		  reference
		  status summary
		  title
		  unexpected))
{
    $expected{$i} = 1;
}

my @soms = soapretrieveall($site, $user, $password, 'Knowledge');
my $id;

my %doctype = (
    errormsg => 'Error message',
    external => 'External document',
    howto => 'Question/Answer',
    probsol => 'Problem solution',
    reference => 'Reference',
    );

my $template = "$0";
$template =~ s/pl$/tmpl/;

foreach my $som (@soms) {
    print "dlknowledge [\n";
    soapdata($som, '//RetrieveKnowledgeResponse/model/instance', 0);
    renderknowledge($som);
    print "dlknowledge ]\n";
}

sub setparams {
    my ($som, $tmpl, $root, %unexpected) = @_;
    my $ref = $som->valueof($root);
    my @keys = keys(%$ref);
    foreach my $k (@keys) {
	foreach my $v ($som->valueof("$root/$k")) {
	    if (! defined $v) {
		if ($expected{$k}) {
		    print "was expecting $k (undefined)\n";
		    $tmpl->param($k => '(undefined)');
		} else {
		    print "was not expecting $k (undefined)\n";
		    $unexpected{$k} = '(undefined)';
		}
	    } elsif (ref($v) eq 'HASH') {
		($tmpl, %unexpected) = setparams($som, $tmpl, "$root/$k", %unexpected);
	    } elsif ($v eq '') {
		if ($expected{$k}) {
		    print "was expecting $k (empty)\n";
		    $tmpl->param($k => '(empty)');
		} else {
		    print "was not expecting $k (empty)\n";
		    $unexpected{$k} = '(empty)';
		}
	    } else {
		if ($k eq 'id') {
		    $id = $v;
		} elsif ($k eq 'doctype') {
		    print "setting is$v 1\n";
		    $tmpl->param("is$v" => 1);
		    $tmpl->param(ndoctype => $doctype{$v});
		} elsif ($k eq 'summary') {
		    print "setting hassummary 1\n";
		    $tmpl->param(hassummary => 1);
		}
		if ($expected{$k}) {
		    print "was expecting $k $v\n";
		    $tmpl->param($k => $v);
		} else {
		    print "was not expecting $k $v\n";
		    $unexpected{$k} = $v;
		}
	    }
	}
    }
    return ($tmpl, %unexpected);
}

sub renderknowledge {
    my $som = $_[0];
    my $tmpl = HTML::Template->new(filename => $template);
    my $k;
    foreach $k (keys(%expected)) {
	$tmpl->param($k => '');
    }
    foreach $k (qw(errormsg external howto probsol reference)) {
	$tmpl->param("is$k" => 0);
    }
    my $root = '//RetrieveKnowledgeResponse';
    my %unexpected;
    ($tmpl, %unexpected) = setparams($som, $tmpl, $root, ());
    my @keys = sort(keys(%unexpected));
    if ($#keys >= 0) {
	$tmpl->param(hasunexpected => 1);
	my $unex = '<dl>';
	foreach $k (@keys) {
	    $unex .= "\n<dt>$k</dt><dd>$unexpected{$k}</dd>";
	}
	$unex .= "\n</dl>";
	$tmpl->param(unexpected => $unex);
    }
    my $f;
    open($f, "> $dir/HT-$id.html") or die "$dir/$id.html";
    print $f $tmpl->output . "\n";
    close($f);
}
